@@ -3,7 +3,6 @@
 #endif
 
 #include <errno.h>
-#include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -21,7 +20,7 @@
 int check_memory(void)
 {
     char buf[1024], *ptr1, *ptr2;
-    int free, res;
+    unsigned int free;
 
     /* is the memory file open? */
     if (mem == -1)
